@@ -1,15 +1,35 @@
 from enum import Enum
-from typing import Any, Dict, List, Literal, Optional, Union
+from typing import Any, Dict, List, Literal, Optional, Type, Union
 
 from bson import ObjectId
 from pydantic import BaseModel, EmailStr, Field, validator
 
 from fastel.exceptions import APIException
 
-from .base import BaseCartConfig
+
+class BaseCartConfig(BaseModel):
+    @classmethod
+    def model_with_optional_fields(cls: Type[BaseModel]) -> Type[BaseModel]:
+        """Generate a `BaseModel` class with all the same fields as `model` but as optional"""
+
+        class OptionalModel(cls):  # type: ignore
+            ...
+
+        for field in OptionalModel.__fields__.values():
+            field.required = False
+
+        # for generated schema for example (can be removed)
+        OptionalModel.__name__ = f"Optional{cls.__name__}"
+
+        return OptionalModel
+
+    @classmethod
+    def validate_optional(cls, value: Any) -> BaseModel:
+        return cls.model_with_optional_fields().validate(value)
 
 
 class PaymentSubTypes(str, Enum):
+    atm = "atm"
     credit = "credit"
     cod = "cod"
 
@@ -56,6 +76,23 @@ class CartConfig(BaseCartConfig):
     payment_subtype: PaymentSubTypes = PaymentSubTypes.credit
     logistics_type: LogisticTypes = LogisticTypes.CVS
     logistics_subtype: str = CVSLogisticSubTypes.FAMIC2C
+    logistics_cvs_store_id: Optional[str]
+    # 1: 應稅 2: 零稅率 3: 免稅 4: 特種應稅 9: 混合稅（限B2C）
+    invoice_tax_type: Literal["1", "2", "3", "4", "9"] = "1"
+    invoice_tax_rate: float = 0.05
+    # B2B 統編
+    invoice_uni_no: Optional[str]
+    invoice_category: Literal["B2B", "B2C"] = "B2C"
+    # B2C 是否捐贈
+    invoice_donation: Literal["0", "1"] = "0"
+    # B2C 捐贈碼
+    invoice_love_code: Optional[str]
+    # B2C 載具類型 0: 無載具 1: 綠界電子發票（使用者email-優先 or phone） 2: 自然人憑證 3: 手機條碼
+    invoice_carrier_type: Literal["0", "1", "2", "3"] = "1"
+    # B2C 載具編號 如果 carrier_type = "1" 載具編號放空字串即可， ecpay 會拿 customer email or phone
+    invoice_carrier_num: str = ""
+    order_note: str = ""
+    extra_data: Optional[Any]
 
     @validator("logistics_subtype")
     def validate_logistic_subtype(cls, value: str, values: Dict[str, Any]) -> str:
@@ -84,7 +121,8 @@ class BoolConfig(BaseModel):
 
 class ItemConfig(BaseModel):
     qty: int
-    variants: List[Union[SingleConfig, BoolConfig]]
+    variants: List[Union[SingleConfig, BoolConfig]] = []
+    extra_data: Optional[Any]
 
     @validator("qty")
     def positive_qty(cls, qty: int) -> int:
@@ -150,7 +188,11 @@ class Product(BaseModel):
     name: str
     price: int
     variants: List[Union[SingleVariant, BooleanVariant]] = []
-    on_shelf: bool = True
+    on_shelf: Optional[bool]
+    public: Optional[bool]
+    labels: Optional[List[str]]
+    images: Optional[Union[List[str], List[Dict[str, Any]]]]
+    extra_data: Optional[Any]
 
 
 class Coupon(BaseModel):
